==> spread-5.0.0.54-py3-none-any.whl: spread_core/tools/statistic.py <==
import base64
import datetime
import json
import logging
import threading
import zlib

import spread_core.mqtt as mqtt
from spread_core.tools import frontier_launcher as frontier
from spread_core.bam import engineries, generator
from spread_core.bam.equipment import RECIPE, INGREDIENTS, TYPE
from spread_core.errors.project_errors import ProjectError
from spread_core.tools.service_launcher import Launcher


class Statistic(Launcher):
    key = '{00000000-0000-0000-0000-000000000000}'
    _dumped = False

    def __init__(self):
        self._manager = self
        self._engineries = []
        self.ids = []
        self.timer = None
        self.cur_e = None
        super(Statistic, self).__init__()

    def start(self):
        self.subscribe(mqtt.project.TopicProject.engineries(self.project_id))

    def on_project(self, data):
        data = json.loads(data)
        if mqtt.project.TopicProject.ENGINERIES in data:
            try:
                self.unsubscribe(mqtt.project.TopicProject.engineries(self.project_id))
                for e_data in data[mqtt.project.TopicProject.ENGINERIES]:
                    if RECIPE in e_data and TYPE in e_data[RECIPE] and INGREDIENTS in e_data[RECIPE]:
                        if e_data[RECIPE][TYPE] == 'Simple' and len(e_data[RECIPE][INGREDIENTS]) > 0:
                            if e_data[TYPE] == engineries.DimmingLight.__name__:
                                try:
                                    _eng = generator.generate_enginery(self.project_id, e_data)
                                    self._engineries.append(_eng)
                                    self.ids.append(_eng.id)
                                except ProjectError as ex:
                                    logging.warning(ex)
            except BaseException as ex:
                logging.exception(ex)
            finally:
                self.on_ready()

    def start_timer(self):
        self.stop_timer()
        self.timer = threading.Timer(5, self.on_time_out)
        self.timer.start()

    def stop_timer(self):
        if self.timer:
            self.timer.cancel()
            self.timer = None

    def on_time_out(self):
        pass
        # logging.info(f'{self.cur_e}: unknown')

    def on_ready(self):
        logging.info('Project({}) loaded'.format(self.project_id))

        class_id = frontier.Frontier.funit_ids[engineries.light.DimmingLight.__name__].index(frontier.historyResponseId)
        topic = f'Tros3/Reply/{self.project_id}/{self.key}/Equipment/DimmingLight/+/{class_id}'
        self.subscribe(topic)
        self.req()

    def req(self):
        if len(self._engineries) > 0:
            self.cur_e = self._engineries.pop(0)
            class_id = frontier.Frontier.funit_ids[engineries.light.DimmingLight.__name__].index(frontier.historyRequestId)
            topic = f'Tros3/Command/{self.project_id}/{self.key}/Equipment/DimmingLight/{self.cur_e.id}/{class_id}'
            data = {
                "algorithm": "raw",
                "startTime": "2021-01-01T00:00:00",
                "endTime": "2021-01-31T00:00:00",
                "key": f'{self.key}'
            }

            var = mqtt.tros3.variable.VariableTRS3(id=self.cur_e.id, cl=class_id, val=json.dumps(data))
            try:
                self.start_timer()
                self.publish(topic, var)
            except BaseException as ex:
                logging.exception(ex)
        else:
            self.on_exit(0, None)

    def on_reply(self, variable):
        if variable.id in self.ids:
            try:
                if self.cur_e and variable.id == self.cur_e.id:
                    self.stop_timer()
                response = json.loads(variable.value)

                period = 0
                power = 0
                if response['data']:
                    static = base64.b64decode(response['data'])
                    static = zlib.decompress(static).decode()
                    static = static.split()

                    last_time = None

                    last_power = 0
                    while len(static) > 0:
                        try:
                            item = static.pop(0).split(',')
                            time = datetime.datetime.fromisoformat(item[1].replace('"', ''))
                            valid = bool(int(item[2]))
                            if valid:
                                if last_power > 0:
                                    gap = round(time.timestamp() - last_time.timestamp())
                                    period += gap
                                    power += last_power * gap

                                last_power = int(item[3])

                            last_time = time

                        except BaseException as ex:
                            logging.exception(ex)

            except BaseException as ex:
                logging.exception(ex)
            else:
                if period > 0 and self.cur_e.power > 0:
                    power = round(power / period / self.cur_e.power * 100)
                else:
                    power = 0
                logging.info(f'{variable.id}:'
                             f' Br={round(power)}%;'.ljust(8, ' ') +
                             f' Tp={round(period / 60 / 60)} ч;'
                             f' P={self.cur_e.power} Вт;')

            self.req()

    def on_message(self, mosq, obj, msg):
        if msg.topic.startswith(f'Tros3/Reply/'):
            var = mqtt.tros3.variable.VariableTRS3.of(msg.payload)
            self.on_reply(var)
        else:
            topic = mqtt.of(msg.topic)
            if isinstance(topic, mqtt.project.TopicProject):
                self.on_project(msg.payload.decode())


if __name__ == '__main__':
    Statistic()

==> spread-5.1.0.48-py3-none-any.whl: spread_core/tools/statistic.py <==
import base64
import datetime
import json
import logging
import threading
import zlib

import spread_core.mqtt as mqtt
from spread_core import launcher_frontier as frontier
from spread_core.bam import engineries, generator
from spread_core.bam.equipment import RECIPE, INGREDIENTS, TYPE
from spread_core.errors.project_errors import ProjectError
from spread_core.tools.service_launcher import Launcher


class Statistic(Launcher):
    key = '{00000000-0000-0000-0000-000000000000}'
    _dumped = False

    def __init__(self):
        self._manager = self
        self._engineries = []
        self.ids = []
        self.timer = None
        self.cur_e = None
        super(Statistic, self).__init__()

    def start(self):
        self.subscribe(mqtt.project.TopicProject.engineries(self.project_id))

    def on_project(self, data):
        data = json.loads(data)
        if mqtt.project.TopicProject.ENGINERIES in data:
            try:
                self.unsubscribe(mqtt.project.TopicProject.engineries(self.project_id))
                for e_data in data[mqtt.project.TopicProject.ENGINERIES]:
                    if RECIPE in e_data and TYPE in e_data[RECIPE] and INGREDIENTS in e_data[RECIPE]:
                        if e_data[RECIPE][TYPE] == 'Simple' and len(e_data[RECIPE][INGREDIENTS]) > 0:
                            if e_data[TYPE] == engineries.DimmingLight.__name__:
                                try:
                                    _eng = generator.generate_enginery(self.project_id, e_data)
                                    self._engineries.append(_eng)
                                    self.ids.append(_eng.id)
                                except ProjectError as ex:
                                    logging.warning(ex)
            except BaseException as ex:
                logging.exception(ex)
            finally:
                self.on_ready()

    def start_timer(self):
        self.stop_timer()
        self.timer = threading.Timer(5, self.on_time_out)
        self.timer.start()

    def stop_timer(self):
        if self.timer:
            self.timer.cancel()
            self.timer = None

    def on_time_out(self):
        pass
        # logging.info(f'{self.cur_e}: unknown')

    def on_ready(self):
        logging.info('Project({}) loaded'.format(self.project_id))

        class_id = frontier.Frontier.funit_ids[engineries.light.DimmingLight.__name__].index(frontier.historyResponseId)
        topic = f'Tros3/Reply/{self.project_id}/{self.key}/Equipment/DimmingLight/+/{class_id}'
        self.subscribe(topic)
        self.req()

    def req(self):
        if len(self._engineries) > 0:
            self.cur_e = self._engineries.pop(0)
            class_id = frontier.Frontier.funit_ids[engineries.light.DimmingLight.__name__].index(frontier.historyRequestId)
            topic = f'Tros3/Command/{self.project_id}/{self.key}/Equipment/DimmingLight/{self.cur_e.id}/{class_id}'
            data = {
                "algorithm": "raw",
                "startTime": "2021-01-01T00:00:00",
                "endTime": "2021-01-31T00:00:00",
                "key": f'{self.key}'
            }

            var = mqtt.tros3.variable.VariableTRS3(id=self.cur_e.id, cl=class_id, val=json.dumps(data))
            try:
                self.start_timer()
                self.publish(topic, var)
            except BaseException as ex:
                logging.exception(ex)
        else:
            self.on_exit(0, None)

    def on_reply(self, variable):
        if variable.id in self.ids:
            try:
                if self.cur_e and variable.id == self.cur_e.id:
                    self.stop_timer()
                response = json.loads(variable.value)

                period = 0
                power = 0
                if response['data']:
                    static = base64.b64decode(response['data'])
                    static = zlib.decompress(static).decode()
                    static = static.split()

                    last_time = None

                    last_power = 0
                    while len(static) > 0:
                        try:
                            item = static.pop(0).split(',')
                            time = datetime.datetime.fromisoformat(item[1].replace('"', ''))
                            valid = bool(int(item[2]))
                            if valid:
                                if last_power > 0:
                                    gap = round(time.timestamp() - last_time.timestamp())
                                    period += gap
                                    power += last_power * gap

                                last_power = int(item[3])

                            last_time = time

                        except BaseException as ex:
                            logging.exception(ex)

            except BaseException as ex:
                logging.exception(ex)
            else:
                if period > 0 and self.cur_e.power > 0:
                    power = round(power / period / self.cur_e.power * 100)
                else:
                    power = 0
                logging.info(f'{variable.id}:'
                             f' Br={round(power)}%;'.ljust(8, ' ') +
                             f' Tp={round(period / 60 / 60)} ч;'
                             f' P={self.cur_e.power} Вт;')

            self.req()

    def on_message(self, mosq, obj, msg):
        if msg.topic.startswith(f'Tros3/Reply/'):
            var = mqtt.tros3.variable.VariableTRS3.of(msg.payload)
            self.on_reply(var)
        else:
            topic = mqtt.of(msg.topic)
            if isinstance(topic, mqtt.project.TopicProject):
                self.on_project(msg.payload.decode())


if __name__ == '__main__':
    Statistic()
